using System;
using System.Data;
using System.Windows.Forms;

namespace BandLoadingBug
{
    using System.Threading;

    public partial class Form1 : Form
    {
        private MyUltraDataSource ds;
        public const string ChildNamePrefix = "Child";
        public const string ChildColumnPrefix = "ChildColumn";
        private const int TableCount = 16;
        private const int ParentColumnCount = 70;
        private const int ChildColumnCount = 20;
        
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            InitDataSource();
        }

        private void InitDataSource()
        {
            DataSet dataSet = new DataSet();
            for (int i = 0; i < TableCount; i++)
            {
                DataTable table = new DataTable("Table" + i);
                dataSet.Tables.Add(table);

                DataColumn idColumn = table.Columns.Add("ID");
                for (int j = 0; j < ParentColumnCount; j++)
                {
                    table.Columns.Add("Column" + j, typeof(string));
                }

                table.PrimaryKey = new [] { idColumn };

                DataTable childTable = new DataTable(ChildNamePrefix + "Table" + i);
                dataSet.Tables.Add(childTable);
                DataColumn childIdColumn = childTable.Columns.Add("ID");
                for (int childColumnIndex = 0; childColumnIndex < ChildColumnCount; childColumnIndex++)
                {
                    childTable.Columns.Add(ChildColumnPrefix + childColumnIndex, typeof(string));
                }
                childTable.PrimaryKey = new [] { childIdColumn };
            }

            ds = new MyUltraDataSource(dataSet);
            ultraGrid1.DataSource = ds;
            //ultraGrid1.BeforeRowExpanded += ds.BeforeRowExpanded;

            Thread thread = new Thread(PopulateDataSource);
            thread.Start();
        }

        private void PopulateDataSource()
        {

            //uncomment these lines to invoke on the GUI thread and the exception does not get thrown
//            ultraGrid1.Invoke((Action)delegate
//                                          {
                                              ultraGrid1.BeginUpdate();
                                              ultraGrid1.SuspendRowSynchronization();

                                              int uniqueRowKey = 0;
                                              DataSet updateSet = new DataSet();
                                              for (int tableIndex = 0; tableIndex < TableCount; tableIndex++)
                                              {
                                                  ParentBody(updateSet, tableIndex, ref uniqueRowKey);
                                              }

                                              ds.UpdateData(updateSet);
                                              ultraGrid1.ResumeRowSynchronization();
                                              ultraGrid1.EndUpdate();
//                                          });
        }


        private void ParentBody(DataSet updateSet, int tableIndex, ref int uniqueRowKey)
        {
            DataTable table = new DataTable("Table" + tableIndex);
            updateSet.Tables.Add(table);
            table.Columns.Add("ID");
            for (int columnIndex = 0; columnIndex < ParentColumnCount; columnIndex++)
            {
                table.Columns.Add("Column" + columnIndex, typeof (string));
            }

            //set up child items
            DataTable childtable = new DataTable(ChildNamePrefix + "Table" + tableIndex);
            updateSet.Tables.Add(childtable);
            childtable.Columns.Add("ID");
            for (int childColumnIndex = 0; childColumnIndex < ChildColumnCount; childColumnIndex++)
            {
                childtable.Columns.Add(ChildColumnPrefix + childColumnIndex, typeof (string));
            }

            object[] data = new object[table.Columns.Count];
            for (int dataIndex = 2; dataIndex < data.Length; dataIndex++)
            {
                data[dataIndex] = String.Format("Table Index={0} Data Index={1}", tableIndex, dataIndex);
            }

            bool childRowNeeded = false;
            for (int rowIndex = 0; rowIndex < 2; rowIndex++)
            {
                //set the id which links row to its child
                data[0] = uniqueRowKey++;
                table.Rows.Add(data);

                if (childRowNeeded)
                {
                    object[] childData = new object[childtable.Columns.Count];
                    childData[0] = data[0];
                    for (int childDataIndex = 1; childDataIndex < childData.Length; childDataIndex++)
                    {
                        childData[childDataIndex] = String.Format("ChildTable Index={0} ChildData Index={1}",
                                                                  childDataIndex, childDataIndex);
                    }

                    childtable.Rows.Add(childData);
                }
                childRowNeeded = !childRowNeeded;
            }
        }
    }
}
